@@ -524,6 +524,17 @@ enum tevent_trace_point {
 	 * Corresponds to a trace point just after waiting
 	 */
 	TEVENT_TRACE_AFTER_WAIT,
+#define TEVENT_HAS_LOOP_ONCE_TRACE_POINTS 1
+	/**
+	 * Corresponds to a trace point just before calling
+	 * the loop_once() backend function.
+	 */
+	TEVENT_TRACE_BEFORE_LOOP_ONCE,
+	/**
+	 * Corresponds to a trace point right after the
+	 * loop_once() backend function has returned.
+	 */
+	TEVENT_TRACE_AFTER_LOOP_ONCE,
 };
 
 typedef void (*tevent_trace_callback_t)(enum tevent_trace_point,
@@ -908,9 +919,9 @@ bool _tevent_req_cancel(struct tevent_req *req, const char *location);
  * req = tevent_req_create(mem_ctx, &state, struct computation_state);
  * @endcode
  *
- * Tevent_req_create() creates the state variable as a talloc child of
- * its result. The state variable should be used as the talloc parent
- * for all temporary variables that are allocated during the async
+ * Tevent_req_create() allocates and zeros the state variable as a talloc
+ * child of its result. The state variable should be used as the talloc
+ * parent for all temporary variables that are allocated during the async
  * computation. This way, when the user of the async computation frees
  * the request, the state as a talloc child will be free'd along with
  * all the temporary variables hanging off the state.
