@@ -1,4 +1,4 @@
-/* 
+/*
    Unix SMB/CIFS implementation.
 
    common events code for signal events
@@ -30,8 +30,6 @@
 #include "tevent_internal.h"
 #include "tevent_util.h"
 
-#define TEVENT_NUM_SIGNALS 64
-
 /* maximum number of SA_SIGINFO signals to hold in the queue.
   NB. This *MUST* be a power of 2, in order for the ring buffer
   wrap to work correctly. Thanks to Petr Vandrovec <petr@vandrovec.name>
@@ -121,10 +119,39 @@ static void tevent_common_signal_handler_info(int signum, siginfo_t *info,
 	if (count+1 == TEVENT_SA_INFO_QUEUE_COUNT) {
 		/* we've filled the info array - block this signal until
 		   these ones are delivered */
+#ifdef HAVE_UCONTEXT_T
+		/*
+		 * This is the only way for this to work.
+		 * By default signum is blocked inside this
+		 * signal handler using a temporary mask,
+		 * but what we really need to do now is
+		 * block it in the callers mask, so it
+		 * stays blocked when the temporary signal
+		 * handler mask is replaced when we return
+		 * from here. The callers mask can be found
+		 * in the ucontext_t passed in as the
+		 * void *uctx argument.
+		 */
+		ucontext_t *ucp = (ucontext_t *)uctx;
+		sigaddset(&ucp->uc_sigmask, signum);
+#else
+		/*
+		 * WARNING !!! WARNING !!!!
+		 *
+		 * This code doesn't work.
+		 * By default signum is blocked inside this
+		 * signal handler, but calling sigprocmask
+		 * modifies the temporary signal mask being
+		 * used *inside* this handler, which will be
+		 * replaced by the callers signal mask once
+		 * we return from here. See Samba
+		 * bug #9550 for details.
+		 */
 		sigset_t set;
 		sigemptyset(&set);
 		sigaddset(&set, signum);
 		sigprocmask(SIG_BLOCK, &set, NULL);
+#endif
 		TEVENT_SIG_INCREMENT(sig_state->sig_blocked[signum]);
 	}
 }
@@ -148,7 +175,19 @@ static int tevent_signal_destructor(struct tevent_signal *se)
 			     struct tevent_common_signal_list);
 
 	if (se->event_ctx) {
-		DLIST_REMOVE(se->event_ctx->signal_events, se);
+		struct tevent_context *ev = se->event_ctx;
+
+		DLIST_REMOVE(ev->signal_events, se);
+
+		if (ev->signal_events == NULL && ev->pipe_fde != NULL) {
+			/*
+			 * This was the last signal. Destroy the pipe.
+			 */
+			TALLOC_FREE(ev->pipe_fde);
+
+			close(ev->pipe_fds[0]);
+			close(ev->pipe_fds[1]);
+		}
 	}
 
 	talloc_free(sl);
@@ -284,7 +323,7 @@ struct tevent_signal *tevent_common_add_signal(struct tevent_context *ev,
 		sig_state->oldact[signum] = talloc(sig_state, struct sigaction);
 		if (sig_state->oldact[signum] == NULL) {
 			talloc_free(se);
-			return NULL;			
+			return NULL;
 		}
 		if (sigaction(signum, &act, sig_state->oldact[signum]) == -1) {
 			talloc_free(se);
@@ -328,7 +367,7 @@ int tevent_common_check_signal(struct tevent_context *ev)
 	if (!sig_state || !TEVENT_SIG_PENDING(sig_state->got_signal)) {
 		return 0;
 	}
-	
+
 	for (i=0;i<TEVENT_NUM_SIGNALS+1;i++) {
 		struct tevent_common_signal_list *sl, *next;
 		struct tevent_sigcounter counter = sig_state->signal_count[i];
@@ -346,6 +385,8 @@ int tevent_common_check_signal(struct tevent_context *ev)
 			struct tevent_signal *se = sl->se;
 			struct tevent_se_exists *exists;
 
+			next = sl->next;
+
 			/*
 			 * We have to be careful to not touch "se"
 			 * after it was deleted in its handler. Thus
@@ -361,7 +402,6 @@ int tevent_common_check_signal(struct tevent_context *ev)
 			talloc_set_destructor(
 				exists, tevent_se_exists_destructor);
 
-			next = sl->next;
 #ifdef SA_SIGINFO
 			if (se->sa_flags & SA_SIGINFO) {
 				uint32_t j;
@@ -376,7 +416,7 @@ int tevent_common_check_signal(struct tevent_context *ev)
 					uint32_t ofs = (counter.seen + j)
 						% TEVENT_SA_INFO_QUEUE_COUNT;
 					se->handler(ev, se, i, 1,
-						    (void*)&sig_state->sig_info[i][ofs], 
+						    (void*)&sig_state->sig_info[i][ofs],
 						    se->private_data);
 					if (!exists) {
 						break;
